'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var common = require('@lezer/common');
var lezerJulia = require('lezer-julia');
var language$1 = require('@codemirror/language');
var autocomplete = require('@codemirror/autocomplete');
var highlight = require('@codemirror/highlight');

function _interopNamespace(e) {
    if (e && e.__esModule) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n["default"] = e;
    return Object.freeze(n);
}

var autocomplete__namespace = /*#__PURE__*/_interopNamespace(autocomplete);
var highlight__namespace = /*#__PURE__*/_interopNamespace(highlight);

function delimitedStrategy(context, units, closing) {
    let after = context.textAfter;
    let space = after.match(/^\s*/)[0].length;
    let closed = false;
    switch (closing.length) {
        case 1:
            closed = after.slice(space, space + closing[0].length) === closing[0];
            break;
        case 2:
            closed =
                after.slice(space, space + closing[0].length) === closing[0] ||
                    after.slice(space, space + closing[1].length) === closing[1];
            break;
        case 3:
            closed =
                after.slice(space, space + closing[0].length) === closing[0] ||
                    after.slice(space, space + closing[1].length) === closing[1] ||
                    after.slice(space, space + closing[2].length) === closing[2];
            break;
        default:
            closed = closing.some((closing) => after.slice(space, space + closing.length) === closing);
            break;
    }
    return context.baseIndent + (closed ? 0 : context.unit * units);
}
function delimitedIndent({ closing, units = 1, }) {
    return (context) => delimitedStrategy(context, units, closing);
}
function noIndent(context) {
    return context.baseIndent;
}

function getSyntaxConfig() {
    let syntaxConfig = {
        indents: {
            VariableDeclaration: language$1.continuedIndent(),
            AssignmentExpression: language$1.continuedIndent(),
        },
        keywords: [],
    };
    for (let node of lezerJulia.parser.nodeSet.types) {
        // Collect keywords
        let groups = node.prop(common.NodeProp.group);
        let group = groups != null ? groups[0] : null;
        if (group === "keyword") {
            syntaxConfig.keywords.push(node);
        }
        // Configure indents
        let nodeIndent;
        let closedBy = node.prop(common.NodeProp.closedBy);
        if (closedBy) {
            nodeIndent = delimitedIndent({ closing: closedBy });
        }
        else {
            nodeIndent = noIndent;
        }
        syntaxConfig.indents[node.name] = nodeIndent;
    }
    return syntaxConfig;
}
let syntaxConfig = getSyntaxConfig();
let styleTags = highlight__namespace.styleTags({
    Identifier: highlight__namespace.tags.variableName,
    String: highlight__namespace.tags.string,
    TripleString: highlight__namespace.tags.string,
    Comment: highlight__namespace.tags.lineComment,
    BlockComment: highlight__namespace.tags.comment,
    [syntaxConfig.keywords.map((t) => t.name).join(" ")]: highlight__namespace.tags.keyword,
    "( )": highlight__namespace.tags.paren,
    "[ ]": highlight__namespace.tags.paren,
    "{ }": highlight__namespace.tags.paren,
    MacroIdentifier: highlight__namespace.tags.macroName,
});
let language = language$1.LRLanguage.define({
    parser: lezerJulia.parser.configure({
        props: [
            styleTags,
            language$1.indentNodeProp.add(Object.assign(Object.assign({}, syntaxConfig.indents), { ModuleDefinition: noIndent, BareModuleDefinition: noIndent, VariableDeclaration: language$1.continuedIndent(), AssignmentExpression: language$1.continuedIndent() })),
        ],
    }),
    languageData: {
        commentTokens: { line: "#" },
        indentOnInput: /^\s*(\]|\}|\)|end|else|elseif|catch|finally)/,
        closeBrackets: { brackets: ["(", "[", "{", "'", '"', "`"] },
    },
});
const keywordCompletion = language.data.of({
    autocomplete: autocomplete__namespace.completeFromList(syntaxConfig.keywords.map((keyword) => ({
        label: keyword.name,
        type: "keyword",
    }))),
});
let defaultConfig = {
    enableKeywordCompletion: false,
};
function julia(config = defaultConfig) {
    config = Object.assign(Object.assign({}, defaultConfig), config);
    let extensions = [];
    if (config.enableKeywordCompletion) {
        extensions.push(keywordCompletion);
    }
    return new language$1.LanguageSupport(language, extensions);
}

exports.julia = julia;
exports.keywordCompletion = keywordCompletion;
